/* LatVis
 * Copyright (C) 2002 Jan Outrata <outrataj@phoenix.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

// class of the generated diagram using the geometric method

#ifndef __GEOMETRICDIAGRAM_H__
#define __GEOMETRICDIAGRAM_H__

#include "diagram.h"

class GeometricDiagram : public Diagram {
 public:
  struct generationParams_t : public Diagram::generationParams_t {
    bool from_greatest;
  };
 protected:
  struct GeomDiagramVertex {
    ElementId element;
    int lower_count, upper_count;
    // comparison function for STL
    bool
    operator <(const GeomDiagramVertex &v) const
    {
      return (element < v.element);
    }
  };
  OrderData *data;
  set<Vertex_e> verteces;
  generationParams_t *generationParams;

  void placeVertexRuleOfParallelograms(Vertex_e &vertex);
  // places the vertex into the diagram under the rule of parallelograms
  void placeVertexRuleOfLines(Vertex_e &vertex);
  // places the vertex into the diagram under the rule of lines
  bool isVertexPlaceEmpty(Coord_e_t x, Coord_e_t y);
  // returns: if the vertex place at (x, y) is empty (ie. if there is 
  // not already a vertex)
 public:
  GeometricDiagram(OrderData *d) { if (d) data = d; }
  ~GeometricDiagram() {}
  void generateDiagram(Diagram::generationParams_t *params);
  void getDiagram(set<Vertex_e> &s) { s = verteces; }
};

#endif
