/* LatVis
 * Copyright (C) 2002 Jan Outrata <outrataj@phoenix.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "orderedset.h"
#include "dialogs.h"
#include "diagramarea.h"
#include "orderedsets.h"
#include "diagramwindow.h"
#include "loader.h"
#include "saver.h"
#include "i18n.h"
#include <gtk--/label.h>
#include <gtk--/box.h>
#include <gtk--/buttonbox.h>
#include <gtk--/frame.h>
#include <gtk--/listitem.h>
#include <gtk--/scrolledwindow.h>
#include <stdio.h>

MessageDialog::MessageDialog(string title, string message)
  : DialogExec(true) // modal
{
  Gtk::Button *ok_button;
  Gtk::Label *label;
  Gtk::HBox *aarea;
  Gtk::VBox *vbox;
  
  set_title(title);
  set_border_width(5);
  ok_button = SigC::manage(new Gtk::Button(_("OK")));
  // can default
  ok_button->set_flags(GTK_CAN_DEFAULT);
  ok_button->clicked.
    connect(SigC::slot(*this, &MessageDialog::accept));
  ok_button->show();
  label = SigC::manage(new Gtk::Label(message));
  // left alignment
  label->set_justify(GTK_JUSTIFY_CENTER);
  label->show();
  aarea = get_action_area();
  aarea->set_homogeneous(true);
  aarea->pack_start(*ok_button, true, true);
  vbox = get_vbox();
  vbox->pack_start(*label, true, false, 5);
  // default
  ok_button->grab_default();
  // disable shrink/growing
  set_policy(false, false, true);
}

QuestionDialog::QuestionDialog(string title, string message, string question)
  : DialogExec(true) // modal
{
  Gtk::Button *ok_button, *cancel_button;
  Gtk::Label *label_m, *label_q;
  Gtk::HBox *aarea;
  Gtk::VBox *vbox;
  
  set_title(title);
  set_border_width(5);
  ok_button = SigC::manage(new Gtk::Button(_("OK")));
  // can default
  ok_button->set_flags(GTK_CAN_DEFAULT);
  ok_button->clicked.
    connect(SigC::slot(*this, &QuestionDialog::accept));
  ok_button->show();
  cancel_button = SigC::manage(new Gtk::Button(_("Cancel")));
  // can default
  cancel_button->set_flags(GTK_CAN_DEFAULT);
  cancel_button->clicked.
    connect(SigC::slot(*this, &QuestionDialog::reject));
  cancel_button->show();
  label_m = SigC::manage(new Gtk::Label(message));
  // left alignment
  label_m->set_justify(GTK_JUSTIFY_CENTER);
  label_m->show();
  label_q = SigC::manage(new Gtk::Label(question));
  // left alignment
  label_q->set_justify(GTK_JUSTIFY_CENTER);
  label_q->show();
  aarea = get_action_area();
  aarea->set_homogeneous(true);
  aarea->pack_start(*ok_button, true, true);
  aarea->pack_start(*cancel_button, true, true);
  vbox = get_vbox();
  vbox->pack_start(*label_m, true, false);
  vbox->pack_start(*label_q, true, false, 5);
  // default
  ok_button->grab_default();
  // disable shrink/growing
  set_policy(false, false, true);
}

void
NewElementDialog::accept()
{
  //if (entry->get_text() == "") return;
  DialogExec::accept();
}

void
NewElementDialog::nameChanged()
{
  //ok_button->set_sensitive((entry->get_text() != ""));
}

NewElementDialog::NewElementDialog(bool rename_dialog)
  : DialogExec(true) // modal
{
  Gtk::Button *cancel_button;
  Gtk::Label *label;
  Gtk::HBox *aarea;
  Gtk::VBox *vbox;
  
  if (!rename_dialog) set_title(_("New element"));
  else set_title(_("Rename element"));
  set_border_width(5);
  if (!rename_dialog) 
    label = SigC::manage(new Gtk::Label(_("New element's name: ")));
  else
    label = SigC::manage(new Gtk::Label(_("Element's new name: ")));
  // left alignment
  label->set_justify(GTK_JUSTIFY_LEFT);
  label->show();
  entry = SigC::manage(new Gtk::Entry);
  // enter
  entry->activate.
    connect(SigC::slot(*this, &NewElementDialog::accept));
  // changed
  entry->changed.
    connect(SigC::slot(*this, &NewElementDialog::nameChanged));
  entry->show();
  ok_button = SigC::manage(new Gtk::Button(_("OK")));
  // can default
  ok_button->set_flags(GTK_CAN_DEFAULT);
  // disabled
  //ok_button->set_sensitive(false);
  ok_button->clicked.
    connect(SigC::slot(*this, &NewElementDialog::accept));
  ok_button->show();
  cancel_button = SigC::manage(new Gtk::Button(_("Cancel")));
  // can default
  cancel_button->set_flags(GTK_CAN_DEFAULT);
  cancel_button->clicked.
    connect(SigC::slot(*this, &NewElementDialog::reject));
  cancel_button->show();
  aarea = get_action_area();
  aarea->set_homogeneous(true);
  aarea->pack_start(*ok_button, true, true);
  aarea->pack_start(*cancel_button, true, true);
  vbox = get_vbox();
  vbox->pack_start(*label, true, true);
  vbox->pack_start(*entry, true, true, 10);
  // focus
  entry->grab_focus();
  // default
  ok_button->grab_default();
  // disable shrink/growing
  set_policy(false, false, true);
}

string
NewElementDialog::getElementName()
{
  return entry->get_text();
}

void
OpenSaveFileDialog::fileChanged()
{
  // disable ok button
  get_ok_button()->
    set_sensitive((get_selection_entry()->get_text() != ""));
}

void
OpenSaveFileDialog::filetypeChanged(string type)
{
  string filename;
  string::size_type pos;
  
  filetype = type;
  if (type == "") return;
  filename = get_selection_entry()->get_text();
  if (filename == "") return;
  pos = filename.rfind('.');
  if (pos != filename.npos) filename.erase(pos);
  filename.append('.' + type);
  get_selection_entry()->set_text(filename);
}

OpenSaveFileDialog::OpenSaveFileDialog()
  : FileSelectionExec(false) // modeless
{
  Gtk::Label *label;
  Gtk::HBox *hbox;

  // disable ok button
  get_ok_button()->set_sensitive(false);
  get_ok_button()->clicked.
    connect(SigC::slot(*this, &OpenSaveFileDialog::accept));
  get_cancel_button()->clicked.
    connect(SigC::slot(*this, &OpenSaveFileDialog::reject));
  // enter on file
  get_selection_entry()->activate.
    connect(SigC::slot(*this, &OpenSaveFileDialog::accept));
  // file changed
  get_selection_entry()->changed.
    connect(SigC::slot(*this, &OpenSaveFileDialog::fileChanged));
  label =
    SigC::manage(new Gtk::Label(_("File type: ")));
  // left alignment
  label->set_justify(GTK_JUSTIFY_LEFT);
  label->show();
  filetype_menu = SigC::manage(new Gtk::Menu());
  filetype_menu->show();
  filetype_optmenu = SigC::manage(new Gtk::OptionMenu());
  filetype_optmenu->set_menu(filetype_menu);
  filetype_optmenu->set_usize(-1, 28);
  filetype_optmenu->show();
  filetype = "";
  hbox = SigC::manage(new Gtk::HBox(false, 5));
  hbox->pack_start(*label, false, false);
  hbox->pack_start(*filetype_optmenu, true, true);
  hbox->show();
  ((Gtk::VBox *)(get_action_area()->get_parent()))->
    pack_start(*hbox, false, false);
}

string
OpenSaveFileDialog::getFilename()
{
  string filename;
  string selstr = _("Selection: ");
  
  filename = get_selection_entry()->get_text();
  if (filename == "") return "";
  filename = (get_selection_text()->get_text() + "/" + filename).
    erase(0, selstr.size());
  return filename;
}

void
OpenFileDialog::accept()
{
  OrderedSet *os;

  if (openFile(getFilename(), os)) {
    SigC::manage(new DiagramWindow(appwindow, orderedsets, os));
    FileSelectionExec::accept();  
  }
}

OpenFileDialog::OpenFileDialog(AppWindow *appwin, OrderedSets *oss)
{ 
  char type[15] = "";

  appwindow = appwin;
  orderedsets = oss;
  set_title(_("Load set"));
  filetype_menu->items().
    push_back(Gtk::Menu_Helpers::MenuElem
	      (_("By Extension"), SigC::bind<string>
	       (SigC::slot(*this, &OpenFileDialog::filetypeChanged), "")));
  filetype_menu->items().push_back(Gtk::Menu_Helpers::SeparatorElem());
  sprintf(type, _("LAT (%s)"), PROG_NAME);
  filetype_menu->items().
    push_back(Gtk::Menu_Helpers::MenuElem
	      (type, SigC::bind<string>
	       (SigC::slot(*this, &OpenFileDialog::filetypeChanged), "lat")));
  filetype_optmenu->set_history(0);
}

bool
OpenFileDialog::openFile(string filename, OrderedSet *&os)
{
  FileLoader *loader;
  FileLoader::error_t load_err;
  MessageDialog *message_dialog;
  
  if (filename == "") return false;
  loader = new FileLoader(orderedsets);
  if (filetype == "lat")
    load_err = loader->load(filename, FileLoader::XML, os);
  else load_err = loader->load(filename, FileLoader::NOTYPE, os);
  delete loader;
  if (load_err == FileLoader::ETYPE) {
    message_dialog = new MessageDialog(_("Load failed!"), 
				       filename + ": " + 
				       _("Unknown file type."));
    message_dialog->exec();
    delete message_dialog;
  }
  else if (load_err == FileLoader::EACCESS) {
    message_dialog = 
      new MessageDialog(_("Load failed!"), 
			filename + " " + 
			_("is not present or is unreadable."));
    message_dialog->exec();
    delete message_dialog;
  }
  else if (load_err == FileLoader::ELOAD) {
    message_dialog = new MessageDialog(_("Load failed!"), 
				       filename + ": " + _("Load error."));
    message_dialog->exec();
    delete message_dialog;
  }
  return (load_err == FileLoader::EOK);
}

void
SaveAsFileDialog::accept()
{
  string filename;
  FILE *file;
  QuestionDialog *question_dialog;
  bool answer = true;

  filename = getFilename();
  file = fopen(filename.c_str(), "r");
  if (file) {
    question_dialog = new QuestionDialog(_("File exists!"), 
					 filename + " " + _("exists."), 
					 _("Overwrite?"));
    answer = question_dialog->exec();
    delete question_dialog;
    fclose(file);
  }
  if (answer && saveFile(filename)) {
    diagramwindow->update();
    FileSelectionExec::accept();
  }
}

SaveAsFileDialog::SaveAsFileDialog(OrderedSet *os, DiagramWindow *win,
				   AppWindow *appwin)
{
  char type[15] = "";

  orderedset = os;
  diagramwindow = win;
  set_title(_("Save set"));
  filetype_menu->items().
    push_back(Gtk::Menu_Helpers::MenuElem
	      (_("By Extension"), SigC::bind<string>
	       (SigC::slot(*this, &SaveAsFileDialog::filetypeChanged), "")));
  filetype_menu->items().push_back(Gtk::Menu_Helpers::SeparatorElem());
  sprintf(type, _("LAT (%s)"), PROG_NAME);
  filetype_menu->items().
    push_back(Gtk::Menu_Helpers::MenuElem
	      (type, SigC::bind<string>
	       (SigC::slot(*this, &SaveAsFileDialog::filetypeChanged),
		"lat")));
  filetype_menu->items().
    push_back(Gtk::Menu_Helpers::MenuElem
	      (_("MetaPost"), SigC::bind<string>
	       (SigC::slot(*this, &SaveAsFileDialog::filetypeChanged),
		"mp")));
  filetype_menu->items().
    push_back(Gtk::Menu_Helpers::MenuElem
	      (_("Encapsulated PostScript"), SigC::bind<string>
	       (SigC::slot(*this, &SaveAsFileDialog::filetypeChanged),
		"eps")));
  filetype_optmenu->set_history(0);
}

bool
SaveAsFileDialog::saveFile(string filename)
{
  FileSaver *saver;
  FileSaver::error_t save_err;
  MessageDialog *message_dialog;
  
  if (filename == "") return false;
  saver = new FileSaver(orderedset);
  if (filetype == "lat")
    save_err = saver->save(filename, FileSaver::XML);
  else if (filetype == "mp")
    save_err = saver->save(filename, FileSaver::MetaPost);
  else if (filetype == "eps")
    save_err = saver->save(filename, FileSaver::PostScript);
  else save_err = saver->save(filename, FileSaver::NOTYPE);
  delete saver;
  if (save_err == FileSaver::ETYPE) {
    message_dialog = new MessageDialog(_("Save failed!"), 
				       filename + ": " + 
				       _("Unknown file type."));
    message_dialog->exec();
    delete message_dialog;
  }
  else if (save_err == FileSaver::EACCESS) {
    message_dialog = 
      new MessageDialog(_("Save failed!"), 
			filename + " " + _("is unwritable."));
    message_dialog->exec();
    delete message_dialog;
  }
  else if (save_err == FileSaver::ESAVE) {
    message_dialog = new MessageDialog(_("Save failed!"), 
				       filename + ": " + _("Save error."));
    message_dialog->exec();
    delete message_dialog;
  }
  return (save_err == FileSaver::EOK);
}

bool
NewSetDialog::checkValue(string val)
{
  width = 0;
  sscanf(val.c_str(), "%i", &width);
  if (width <= 0) return false;
  return true;
}

void
NewSetDialog::accept()
{
  OrderedSet *os;

  width = height = 0;
  sscanf(entry_w->get_text().c_str(), "%i", &width);
  sscanf(entry_h->get_text().c_str(), "%i", &height);
  if ((width <= 0) || (height <= 0)) return;
  DialogExec::accept();
  if (tree_button->get_active())
    os = orderedsets->newSet(OrderedSet::DATA_TREE);
  else if (list_button->get_active())
    os = orderedsets->newSet(OrderedSet::DATA_LIST);
  else if (array_button->get_active())
    os = orderedsets->newSet(OrderedSet::DATA_ARRAY);
  SigC::manage(new DiagramWindow(appwindow, orderedsets, os, width, height));
}

void
NewSetDialog::sizeChanged()
{
  ok_button->set_sensitive(checkValue(entry_w->get_text()) && 
			   checkValue(entry_h->get_text()));
}

void
NewSetDialog::reset()
{
  char w[5] = "", h[5] = "";

  array_button->set_active(true);
  list_button->set_active(false);
  tree_button->set_active(false);
  width = height = 200;
  sprintf(w, "%i", width);
  entry_w->set_text(w);
  sprintf(h, "%i", height);
  entry_h->set_text(h);
}

NewSetDialog::NewSetDialog(AppWindow *appwin, OrderedSets *oss)
  : DialogExec(false) // modeless
{
  Gtk::Button *cancel_button, *reset_button;
  Gtk::Frame *data_frame, *size_frame;
  Gtk::VButtonBox *vbuttonbox;
  Gtk::HBox *aarea, *data_hbox, *width_hbox, *height_hbox, *size_hbox;
  Gtk::VBox *main_vbox, *vbox;
  Gtk::Label *label_w, *label_h;

  appwindow = appwin;
  orderedsets = oss;
  set_title(_("New set"));
  set_border_width(5);
  array_button = 
    SigC::manage(new Gtk::RadioButton(_("Two-dimensional array")));
  array_button->show();
  list_button = SigC::manage(new Gtk::RadioButton(_("List of lists")));
  list_button->set_group(array_button->group());
  list_button->show();
  tree_button = SigC::manage(new Gtk::RadioButton(_("Red-Black tree")));
  tree_button->set_group(array_button->group());
  tree_button->show();
  label_w =
    SigC::manage(new Gtk::Label(_("Width: ")));
  // left alignment
  label_w->set_justify(GTK_JUSTIFY_LEFT);
  label_w->show();
  // max length
  entry_w = SigC::manage(new Gtk::Entry(4));
  // changed
  entry_w->changed.
    connect(SigC::slot(*this, &NewSetDialog::sizeChanged));
  entry_w->show();
  label_h =
    SigC::manage(new Gtk::Label(_("Height: ")));
  // left alignment
  label_h->set_justify(GTK_JUSTIFY_LEFT);
  label_h->show();
  // max length
  entry_h = SigC::manage(new Gtk::Entry(4));
  // changed
  entry_h->changed.
    connect(SigC::slot(*this, &NewSetDialog::sizeChanged));
  entry_h->show();
  ok_button = SigC::manage(new Gtk::Button(_("OK")));
  // can default
  ok_button->set_flags(GTK_CAN_DEFAULT);
  ok_button->clicked.
    connect(SigC::slot(*this, &NewSetDialog::accept));
  ok_button->show();
  reset_button = SigC::manage(new Gtk::Button(_("Reset")));
  // can default
  reset_button->set_flags(GTK_CAN_DEFAULT);
  reset_button->clicked.
    connect(SigC::slot(*this, &NewSetDialog::reset));
  reset_button->show();
  cancel_button = SigC::manage(new Gtk::Button(_("Cancel")));
  // can default
  cancel_button->set_flags(GTK_CAN_DEFAULT);
  cancel_button->clicked.
    connect(SigC::slot(*this, &NewSetDialog::reject));
  cancel_button->show();
  aarea = get_action_area();
  aarea->set_homogeneous(true);
  aarea->pack_start(*ok_button, true, true);
  aarea->pack_start(*reset_button, true, true);
  aarea->pack_start(*cancel_button, true, true);
  vbuttonbox = 
    SigC::manage(new Gtk::VButtonBox(GTK_BUTTONBOX_DEFAULT_STYLE, 0));
  vbuttonbox->set_border_width(10);
  vbuttonbox->pack_start(*array_button, true, true);
  vbuttonbox->pack_start(*list_button, true, true);
  vbuttonbox->pack_start(*tree_button, true, true);
  vbuttonbox->show();
  data_hbox = SigC::manage(new Gtk::HBox());
  data_hbox->pack_start(*vbuttonbox, false, false);
  data_hbox->show();
  data_frame = 
    SigC::manage(new Gtk::Frame(_("Internal data representation")));
  data_frame->add(*data_hbox);
  data_frame->show();
  width_hbox = SigC::manage(new Gtk::HBox(false, 5));
  width_hbox->pack_start(*label_w, false, false);
  width_hbox->pack_start(*entry_w, false, false);
  width_hbox->show();
  height_hbox = SigC::manage(new Gtk::HBox(false, 5));
  height_hbox->pack_start(*label_h, false, false);
  height_hbox->pack_start(*entry_h, false, false);
  height_hbox->show();
  size_hbox = SigC::manage(new Gtk::HBox(true, 10));
  size_hbox->set_border_width(10);
  size_hbox->pack_start(*width_hbox, true, true);
  size_hbox->pack_start(*height_hbox, true, true);
  size_hbox->show();
  size_frame = SigC::manage(new Gtk::Frame(_("Minimal diagram size")));
  size_frame->add(*size_hbox);
  size_frame->show();
  vbox = SigC::manage(new Gtk::VBox(false, 10));
  vbox->set_border_width(10);
  vbox->pack_start(*data_frame, true, true);
  vbox->pack_start(*size_frame, true, true);
  vbox->show();
  main_vbox = get_vbox();
  main_vbox->pack_start(*vbox, true, true);
  // default
  ok_button->grab_default();
  // disable shrink/growing
  set_policy(false, false, true);
  reset();
}

void
RearrangeDiagramDialog::rearrange()
{
  int num = 0;
  bool ok = true;
  MessageDialog message_dialog(_("Warning!"), 
			       _("Maximal width of the Coffman-Graham layering must be set to a positive number."));

  num = notebook->get_current_page_num();
  if (num == 0) {
    generation_params.generation_method = Diagram_Graphic::DIAGRAM_LEVEL;
    generation_params.dle_fmin = level_min_button->get_active();
  }
  else if (num == 1) {
    generation_params.generation_method = 
      Diagram_Graphic::DIAGRAM_GEOMETRIC;
    generation_params.dg_fgreat = geom_great_button->get_active();
  }
  else if (num == 2) {
    generation_params.generation_method = Diagram_Graphic::DIAGRAM_LAYER;
    generation_params.dla_fmax = layer_max_button->get_active();
    if (longest_path_button->get_active())
      generation_params.dla_l = Diagram_Graphic::DLA_L_LONGEST_PATH;
    else if (coffman_graham_button->get_active())
      generation_params.dla_l = Diagram_Graphic::DLA_L_COFFMAN_GRAHAM;
    sscanf(entry->get_text().c_str(), "%i", &num);
    if (num <= 0) { message_dialog.exec(); ok = false; }
    generation_params.dla_cgw = num;
    if (tree_button->get_active())
      generation_params.dla_dsdt = Diagram_Graphic::DLA_DD_TREE;
    else if (list_button->get_active())
      generation_params.dla_dsdt = Diagram_Graphic::DLA_DD_LIST;
    else if (array_button->get_active())
      generation_params.dla_dsdt = Diagram_Graphic::DLA_DD_ARRAY;
    if (adjacent_exchange_button->get_active())
      generation_params.dla_cr = Diagram_Graphic::DLA_CR_ADJACENT_EXCHANGE;
    else if (split_button->get_active())
      generation_params.dla_cr = Diagram_Graphic::DLA_CR_SPLIT;
    else if (barycenter_button->get_active())
      generation_params.dla_cr = Diagram_Graphic::DLA_CR_BARYCENTER;
    else if (median_button->get_active())
      generation_params.dla_cr = Diagram_Graphic::DLA_CR_MEDIAN;
  }
  if (ok) {
    orderedset->save_UndoRedo();
    orderedset->getDiagram()->generateDiagram(generation_params);
    diagramwindow->update();
  }
}

RearrangeDiagramDialog::RearrangeDiagramDialog(OrderedSet *os, 
					       DiagramWindow *win,
					       AppWindow *appwin)
  : DialogExec(false) // modeless
{
  Gtk::Button *rearrange_button, *cancel_button;
  Gtk::Frame *level_frame, *geom_frame, *layer_frame, *l_frame,
    *dsdt_frame, *cr_frame;
  Gtk::VButtonBox *dsdt_vbuttonbox;
  Gtk::HButtonBox *l_hbuttonbox;
  Gtk::HBox *aarea, *level_hbox, *geom_hbox, *layer_hbox, *cgw_hbox,
    *dsdt_hbox, *cr_hbox;
  Gtk::VBox *main_vbox, *vbox, *cr_vbox1, *cr_vbox2, *level_vbox, *geom_vbox,
    *layer_vbox;
  Gtk::Label *label;
  char num[5] = "";
  
  orderedset = os;
  diagramwindow = win;
  generation_params = orderedset->getDiagram()->getGenerationParams();
  set_title(_("Rearrange diagram"));
  set_border_width(5);
  level_min_button = 
    SigC::manage(new Gtk::CheckButton(_("Generate diagram starting with minimal elements")));
  level_min_button->show();
  if (generation_params.dle_fmin) level_min_button->set_active(true);
  geom_great_button = 
    SigC::manage(new Gtk::CheckButton(_("Generate diagram starting with the greatest element")));
  geom_great_button->show();
  if (generation_params.dg_fgreat) geom_great_button->set_active(true);
  layer_max_button = 
    SigC::manage(new Gtk::CheckButton(_("Generate diagram starting with maximal elements")));
  layer_max_button->show();
  if (generation_params.dla_fmax) layer_max_button->set_active(true);
  longest_path_button = SigC::manage(new Gtk::RadioButton(_("Longest Path")));
  longest_path_button->show();
  coffman_graham_button = 
    SigC::manage(new Gtk::RadioButton(_("Coffman-Graham")));
  coffman_graham_button->set_group(longest_path_button->group());  
  coffman_graham_button->show();
  switch (generation_params.dla_l) {
  case Diagram_Graphic::DLA_L_LONGEST_PATH: 
    longest_path_button->set_active(true); break;
  case Diagram_Graphic::DLA_L_COFFMAN_GRAHAM: 
    coffman_graham_button->set_active(true); break;
  }
  label =
    SigC::manage(new Gtk::Label(_("Maximal width of the Coffman-Graham layering: ")));
  // left alignment
  label->set_justify(GTK_JUSTIFY_LEFT);
  label->show();
  // max length
  entry = SigC::manage(new Gtk::Entry(4));
  sprintf(num, "%i", generation_params.dla_cgw);
  entry->set_text(num);
  entry->show();
  array_button = 
    SigC::manage(new Gtk::RadioButton(_("Two-dimensional array")));
  array_button->show();
  list_button = SigC::manage(new Gtk::RadioButton(_("List of lists")));
  list_button->set_group(array_button->group());  
  list_button->show();
  tree_button = SigC::manage(new Gtk::RadioButton(_("Red-Black tree")));
  tree_button->set_group(array_button->group());  
  tree_button->show();
  switch (generation_params.dla_dsdt) {
  case Diagram_Graphic::DLA_DD_TREE: 
    tree_button->set_active(true); break;
  case Diagram_Graphic::DLA_DD_LIST: 
    list_button->set_active(true); break;
  case Diagram_Graphic::DLA_DD_ARRAY: 
    array_button->set_active(true); break;
  }
  adjacent_exchange_button = 
    SigC::manage(new Gtk::RadioButton(_("Adjacent-Exchange sorting")));
  adjacent_exchange_button->show();
  split_button = SigC::manage(new Gtk::RadioButton(_("Split sorting")));
  split_button->set_group(adjacent_exchange_button->group());  
  split_button->show();
  barycenter_button = SigC::manage(new Gtk::RadioButton(_("Barycenter")));
  barycenter_button->set_group(adjacent_exchange_button->group());  
  barycenter_button->show();
  median_button = SigC::manage(new Gtk::RadioButton(_("Median")));
  median_button->set_group(adjacent_exchange_button->group());  
  median_button->show();
  switch (generation_params.dla_cr) {
  case Diagram_Graphic::DLA_CR_ADJACENT_EXCHANGE: 
    adjacent_exchange_button->set_active(true); break;
  case Diagram_Graphic::DLA_CR_SPLIT: 
    split_button->set_active(true); break;
  case Diagram_Graphic::DLA_CR_BARYCENTER: 
    barycenter_button->set_active(true); break;
  case Diagram_Graphic::DLA_CR_MEDIAN: 
    median_button->set_active(true); break;
  }
  rearrange_button = SigC::manage(new Gtk::Button(_("Rearrange")));
  // can default
  rearrange_button->set_flags(GTK_CAN_DEFAULT);
  rearrange_button->clicked.
    connect(SigC::slot(*this, &RearrangeDiagramDialog::rearrange));
  rearrange_button->show();
  cancel_button = SigC::manage(new Gtk::Button(_("Cancel")));
  // can default
  cancel_button->set_flags(GTK_CAN_DEFAULT);
  cancel_button->clicked.
    connect(SigC::slot(*this, &RearrangeDiagramDialog::reject));
  cancel_button->show();
  aarea = get_action_area();
  aarea->set_homogeneous(true);
  aarea->pack_start(*rearrange_button, true, true);
  aarea->pack_start(*cancel_button, true, true);
  level_hbox = SigC::manage(new Gtk::HBox());
  level_hbox->pack_start(*level_min_button, false, false);
  level_hbox->show();
  geom_hbox = SigC::manage(new Gtk::HBox());
  geom_hbox->pack_start(*geom_great_button, false, false);
  geom_hbox->show();
  layer_hbox = SigC::manage(new Gtk::HBox());
  layer_hbox->pack_start(*layer_max_button, false, false);
  layer_hbox->show();
  l_hbuttonbox = 
    SigC::manage(new Gtk::HButtonBox(GTK_BUTTONBOX_DEFAULT_STYLE, 0));
  l_hbuttonbox->set_border_width(10);
  l_hbuttonbox->pack_start(*longest_path_button, true, true);
  l_hbuttonbox->pack_start(*coffman_graham_button, true, true);
  l_hbuttonbox->show();
  cgw_hbox = SigC::manage(new Gtk::HBox(false, 5));
  cgw_hbox->pack_start(*label, false, false);
  cgw_hbox->pack_start(*entry, false, false);
  cgw_hbox->show();
  dsdt_vbuttonbox = 
    SigC::manage(new Gtk::VButtonBox(GTK_BUTTONBOX_DEFAULT_STYLE, 0));
  dsdt_vbuttonbox->set_border_width(10);
  dsdt_vbuttonbox->pack_start(*array_button, true, true);
  dsdt_vbuttonbox->pack_start(*list_button, true, true);
  dsdt_vbuttonbox->pack_start(*tree_button, true, true);
  dsdt_vbuttonbox->show();
  dsdt_hbox = SigC::manage(new Gtk::HBox());
  dsdt_hbox->pack_start(*dsdt_vbuttonbox, false, false);
  dsdt_hbox->show();
  cr_vbox1 = SigC::manage(new Gtk::VBox());
  cr_vbox1->pack_start(*adjacent_exchange_button, true, true);
  cr_vbox1->pack_start(*barycenter_button, true, true);
  cr_vbox1->show();
  cr_vbox2 = SigC::manage(new Gtk::VBox());
  cr_vbox2->pack_start(*split_button, true, true);
  cr_vbox2->pack_start(*median_button, true, true);
  cr_vbox2->show();
  cr_hbox = SigC::manage(new Gtk::HBox());
  cr_hbox->set_border_width(10);
  cr_hbox->pack_start(*cr_vbox1, true, true);
  cr_hbox->pack_start(*cr_vbox2, true, true);
  cr_hbox->show();
  l_frame = SigC::manage(new Gtk::Frame(_("Layering")));
  l_frame->add(*l_hbuttonbox);
  l_frame->show();
  dsdt_frame = 
    SigC::manage(new Gtk::Frame(_("Dummy chains internal data representation")));
  dsdt_frame->add(*dsdt_hbox);
  dsdt_frame->show();
  cr_frame = SigC::manage(new Gtk::Frame(_("Crossing reduction")));
  cr_frame->add(*cr_hbox);
  cr_frame->show();
  level_vbox = SigC::manage(new Gtk::VBox(false, 10));
  level_vbox->set_border_width(10);
  level_vbox->pack_start(*level_hbox, false, false);
  level_vbox->show();
  geom_vbox = SigC::manage(new Gtk::VBox(false, 10));
  geom_vbox->set_border_width(10);
  geom_vbox->pack_start(*geom_hbox, false, false);
  geom_vbox->show();
  layer_vbox = SigC::manage(new Gtk::VBox(false, 10));
  layer_vbox->set_border_width(10);
  layer_vbox->pack_start(*layer_hbox, false, false);
  layer_vbox->pack_start(*l_frame, false, false);
  layer_vbox->pack_start(*cgw_hbox, false, false);
  layer_vbox->pack_start(*dsdt_frame, false, false);
  layer_vbox->pack_start(*cr_frame, false, false);
  layer_vbox->show();
  level_frame = SigC::manage(new Gtk::Frame(_("Level method parameters")));
  level_frame->set_border_width(10);
  level_frame->add(*level_vbox);
  level_frame->show();
  geom_frame = SigC::manage(new Gtk::Frame(_("Geometric method parameters")));
  geom_frame->set_border_width(10);
  geom_frame->add(*geom_vbox);
  geom_frame->show();
  layer_frame = SigC::manage(new Gtk::Frame(_("Layer method parameters")));
  layer_frame->set_border_width(10);
  layer_frame->add(*layer_vbox);
  layer_frame->show();
  notebook = SigC::manage(new Gtk::Notebook());
  // tabs position
  notebook->set_tab_pos(GTK_POS_TOP);
  // popup
  notebook->popup_enable();
  notebook->pages().
    push_back(Gtk::Notebook_Helpers::TabElem(*level_frame, 
					     _("Level method")));
  notebook->pages().
    push_back(Gtk::Notebook_Helpers::TabElem(*geom_frame, 
					     _("Geometric method")));
  notebook->pages().
    push_back(Gtk::Notebook_Helpers::TabElem(*layer_frame, 
					     _("Layer method")));
  // selected method
  switch (generation_params.generation_method) {
  case Diagram_Graphic::DIAGRAM_LEVEL: 
    notebook->set_page(0); break;
  case Diagram_Graphic::DIAGRAM_GEOMETRIC: 
    notebook->set_page(1); break;
  case Diagram_Graphic::DIAGRAM_LAYER: 
    notebook->set_page(2); break;
  }
  notebook->show();
  vbox = SigC::manage(new Gtk::VBox(false, 10));
  vbox->set_border_width(10);
  vbox->pack_start(*notebook, true, true);
  vbox->show();
  main_vbox = get_vbox();
  main_vbox->pack_start(*vbox, true, true);
  // default
  rearrange_button->grab_default();
  // disable shrink/growing
  set_policy(false, false, true);
}

void
SelectPathDialog::select()
{
  Gtk::List::SelectionList &selection = paths_list->selection();
  Gtk::ListItem *item;
  int pos = 0, i;
  list<set<Element> >::iterator iter_l;
  
  if (selection.empty()) return;
  item = *(selection.begin());
  if (!item) return;
  pos = paths_list->child_position(*item);
  for (iter_l = element_paths.begin(), i = 0; i < pos; iter_l++, i++);
  diagramarea->markElements(*iter_l);
  diagramarea->update();
}

SelectPathDialog::SelectPathDialog(DiagramArea *area)
  : DialogExec(false) // modeless
{
  Gtk::Button *cancel_button;
  Gtk::HBox *aarea, *label_hbox;
  Gtk::VBox *main_vbox, *vbox;
  Gtk::Label *label;
  Gtk::ScrolledWindow *scrollwin;

  diagramarea = area;
  set_title(_("Select path"));
  set_border_width(5);
  label =
    SigC::manage(new Gtk::Label(_("Found paths: ")));
  // left alignment
  label->set_justify(GTK_JUSTIFY_LEFT);
  label->show();
  label_hbox = SigC::manage(new Gtk::HBox());
  label_hbox->pack_start(*label, false, false);
  label_hbox->show();
  paths_list = SigC::manage(new Gtk::List());
  paths_list->set_selection_mode(GTK_SELECTION_BROWSE);
  paths_list->selection_changed.
    connect(SigC::slot(*this, &SelectPathDialog::select));
  paths_list->show();
  scrollwin = SigC::manage(new Gtk::ScrolledWindow());
  // automatic scrollbars
  scrollwin->set_policy(GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  scrollwin->set_usize(250, 165);
  scrollwin->add_with_viewport(*paths_list);
  scrollwin->show();
  cancel_button = SigC::manage(new Gtk::Button(_("Cancel")));
  // can default
  cancel_button->set_flags(GTK_CAN_DEFAULT);
  cancel_button->clicked.
    connect(SigC::slot(*this, &SelectPathDialog::reject));
  cancel_button->show();
  aarea = get_action_area();
  aarea->set_homogeneous(true);
  aarea->pack_start(*cancel_button, true, true);
  vbox = SigC::manage(new Gtk::VBox(false, 10));
  vbox->set_border_width(10);
  vbox->pack_start(*label_hbox, false, false);
  vbox->pack_start(*scrollwin, true, true);
  vbox->show();
  main_vbox = get_vbox();
  main_vbox->pack_start(*vbox, true, true);
  // default
  cancel_button->grab_default();
  // disable shrinking
  set_policy(false, true, false);
}

void
SelectPathDialog::setPaths(list<set<Element> > &paths)
{
  list<set<Element> >::iterator iter_l;
  set<Element>::iterator iter_s;
  string text;
  Gtk::ListItem *list_item;

  element_paths = paths;
  paths_list->clear_items(0, -1);
  for (iter_l = paths.begin(); iter_l != paths.end(); iter_l++) {
    text = "";
    for (iter_s = (*iter_l).begin(); iter_s != (*iter_l).end(); iter_s++) {
      if (iter_s != (*iter_l).begin()) text += " - ";
      text += (*iter_s).name;
    }
    if (text != "") {
      list_item = SigC::manage(new Gtk::ListItem(text));
      list_item->show();
      paths_list->add(*list_item);
    }
  }
}

AboutDialog::AboutDialog()
  : DialogExec(false) // modeless
{
  Gtk::VBox *main_vbox, *vbox;
  Gtk::Label *label;
  Gtk::Button *ok_button;
  Gtk::HBox *aarea;

  set_title(_("About the") + string(" ") + string(PROG_NAME));
  set_border_width(5);
  label = 
    SigC::manage(new Gtk::Label(string(PROG_NAME) + "\n\n" + 
				_("Visualize lattices and ordered sets!") + "\n\n" + 
				_("Version") + " " + VERSION + "\n\n" +
				_("Author: ") + "Jan Outrata" + "\n\n" + 
				"Email: outrataj@phoenix.inf.upol.cz"));
  // center alignment
  label->set_justify(GTK_JUSTIFY_CENTER);
  label->show();
  ok_button = SigC::manage(new Gtk::Button(_("OK")));
  // can default
  ok_button->set_flags(GTK_CAN_DEFAULT);
  ok_button->clicked.connect(SigC::slot(*this, &AboutDialog::accept));
  ok_button->show();
  aarea = get_action_area();
  aarea->set_homogeneous(true);
  aarea->pack_start(*ok_button, true, true);
  vbox = SigC::manage(new Gtk::VBox(false, 10));
  vbox->set_border_width(10);
  vbox->pack_start(*label, false, false);
  vbox->show();
  main_vbox = get_vbox();
  main_vbox->pack_start(*vbox, true, true);
  // default
  ok_button->grab_default();
  // disable shrink/growing
  set_policy(false, false, true);
}
