/* LatVis
 * Copyright (C) 2002 Jan Outrata <outrataj@phoenix.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

// class of the ordered set data represented as a list of lists

#ifndef __ORDERDATA_LIST_H__
#define __ORDERDATA_LIST_H__

#include "orderdata.h"

class OrderData_List : public OrderData {
 protected:
  struct ListItem {
    ElementId element;
    set<ElementId> list;
    // comparison function for STL
    bool
    operator <(const ListItem &i) const
    {
      return (element < i.element);
    }
  };
  set<ListItem> list_upper;
  set<ListItem> list_lower;
 protected:
  bool isElement(ElementId x);
  // returns: if x is one of the elements
  bool insertPair(ElementId x, ElementId y);
  // inserts new pair (x, y)
  // returns: if successful (not)
  bool erasePair(ElementId x, ElementId y);
  // erases the pair (x, y)
  // returns: if successful (not)
  bool lessTransitive(ElementId x, ElementId y, bool transitivity = true);
  // if transitivity is false, doesn't iterate transitively
  // returns: if element x is less than element y
  void minimal_maximal(set<ElementId> &s, set<ElementId> ss, bool minimal);
  // returns: minimal (if minimal is true) or maximal elements of the 
  // subset ss in s
  ElementId least_greatest(set<ElementId> ss, bool least);
  // returns: the least (if least is true) or the greatest element of 
  // the subset ss, or NoElement
  void lower_upperCone(set<ElementId> &s, set<ElementId> ss, bool lower);
  // returns: lower (if lower is true) or upper cone of the subset ss in s
  void getPaths_(list<set<ElementId> > &l, set<ElementId> s, ElementId x,
		 ElementId y);
  // returns: the list of all paths from element x to element y in l
 public:
  OrderData_List() {}
  ~OrderData_List();
  OrderData *getCopy();
  void clear();
  bool newElement(ElementId x);
  bool removeElement(ElementId x);
  bool orderElements(ElementId x, ElementId y);
  bool removeOrder(ElementId x, ElementId y);
  bool less(ElementId x, ElementId y, bool transitivity = true);
  void getElements(set<ElementId> &s);
  void upperElements(set<ElementId> &s, ElementId x);
  void lowerElements(set<ElementId> &s, ElementId x);
  void minimalElements(set<ElementId> &s, set<ElementId> ss);
  void maximalElements(set<ElementId> &s, set<ElementId> ss);
  ElementId leastElement(set<ElementId> ss);
  ElementId greatestElement(set<ElementId> ss);
  void lowerCone(set<ElementId> &s, set<ElementId> ss);
  void upperCone(set<ElementId> &s, set<ElementId> ss);
  ElementId infimum(set<ElementId> ss);
  ElementId supremum(set<ElementId> ss);
  void getPaths(list<set<ElementId> > &l, ElementId x, ElementId y);
};

#endif
