#!/bin/sh

datas=("tree" "list" "array")
colors=("red" "green" "blue")
tsts=("newElement" "orderElements" "less" "operations" "LevelDiagram" "GeometricDiagram" "LayerDiagram")
values=("time" "mem")
count=10

perf() {
# data(0-2) n test(0-6)
    ./perf $1 $2 $3 | grep ${tsts[$3]} | \
	awk '{printf("%s %s", $3, $5)}'
}

sq() {
# start step stop
#    seq -f %f $1 $2 $3 | awk -F, '{print $1}'
    seq $1 $2 $3
}

graph() {
# n_min n_max val_min val_max val_scale test_time(0)/mem(1) val_unit
    nmin=$1
    nmax=$2
    nstep=$((($nmax - $nmin) / $count))
    vmin=$3
    vmax=$4
    vstep=$((($vmax - $vmin) / $count))

    cat > $file <<EOF
\input upfigure.mp

beginfig (1)
  dx := 10cm; dy := 5cm;
  ax := $nmin; bx := $nmax;
  ay := $vmin; by := $vmax;
  uy := $5;

  transform t;
  t := _mk_transform;
  
  _draw_arrow_axis(ax, bx, ay, by, t);
  label.lrt (btex \$N\$ etex, (bx, 0) transformed t shifted (3mm, -1mm));
  label.ulft (btex \$$7\$ etex, (0, by) transformed t shifted (-1mm, 3mm));

EOF

    for n in `sq $nmin $nstep $nmax`; do
	echo "  _x_label(btex \$$n\$ etex, $n, t);" >> $file
    done
    for n in `sq $vmin $vstep $vmax`; do
	echo "  _y_label(btex \$$n\$ etex, $n, t);" >> $file
    done
    echo >> $file
    for data in 0 1 2; do
	echo "data:" ${datas[$data]}
	echo "  pair p;" >> $file
	echo "  path p_${datas[$data]};" >> $file
	echo -n "  p_${datas[$data]} := " >> $file
	for n in `sq $nmin $nstep $(($nmax-1))`; do
	    val=(`perf $data $n $tst`)
	    echo -n "($n, ${val[$6]}uy).." >> $file
	done
	val=(`perf $data $nmax $tst`)
	echo "($(($n+$nstep)), ${val[$6]}uy);" >> $file
	echo "  draw p_${datas[$data]} transformed t withcolor ${colors[$data]};" >> $file
	echo "  p := point length (p_${datas[$data]}) of p_${datas[$data]};" >> $file
	echo "  label.rt(btex \$${datas[$data]}\$ etex, p transformed t) withcolor ${colors[$data]};" >> $file
    done
    
    cat >> $file <<EOF

endfig;

bye
EOF
}

do_test()
{
# test(0-6) n_min n_max val_min val_max val_scale test_time(0)/mem(1) val_unit
    tst=$1
    echo "test:" ${tsts[$tst]} "-" ${values[$7]}
    file=${tsts[$tst]}.${values[$7]}.mp
    graph $2 $3 $4 $5 $6 $7 "$8"
}

# test(0-6) n_min n_max val_min val_max val_scale test_time(0)/mem(1) val_unit
#do_test 0 0 500 0 30 10 0 "${values[0]}\ [0.1\ s]"
do_test 0 0 200 0 10 10 0 "${values[0]}\ [0.1\ ms]"
do_test 1 0 100 0 60 100 0 "${values[0]}\ [10\ \mu s]"
do_test 2 0 100 0 30 100 0 "${values[0]}\ [10\ \mu s]"
do_test 3 0 100 0 30 0.01 0 "${values[0]}\ [0.1\ s]"
do_test 0 0 500 0 300 1 1 "${values[1]}\ [kB]"
do_test 1 0 100 0 30 1 1 "${values[1]}\ [kB]"
do_test 4 0 100 0 90 1 0 "${values[0]}\ [ms]"
do_test 5 0 100 0 20 0.01 0 "${values[0]}\ [0.1\ s]"
do_test 6 0 100 0 30 0.01 0 "${values[0]}\ [0.1\ s]"
